@@ -2,7 +2,7 @@
 // detail/reactor_op.hpp
 // ~~~~~~~~~~~~~~~~~~~~~
 //
-// Copyright (c) 2003-2014 Christopher M. Kohlhoff (chris at kohlhoff dot com)
+// Copyright (c) 2003-2023 Christopher M. Kohlhoff (chris at kohlhoff dot com)
 //
 // Distributed under the Boost Software License, Version 1.0. (See accompanying
 // file LICENSE_1_0.txt or copy at http://www.boost.org/LICENSE_1_0.txt)
@@ -30,20 +30,30 @@ public:
   // The error code to be passed to the completion handler.
   asio::error_code ec_;
 
+  // The operation key used for targeted cancellation.
+  void* cancellation_key_;
+
   // The number of bytes transferred, to be passed to the completion handler.
   std::size_t bytes_transferred_;
 
+  // Status returned by perform function. May be used to decide whether it is
+  // worth performing more operations on the descriptor immediately.
+  enum status { not_done, done, done_and_exhausted };
+
   // Perform the operation. Returns true if it is finished.
-  bool perform()
+  status perform()
   {
     return perform_func_(this);
   }
 
 protected:
-  typedef bool (*perform_func_type)(reactor_op*);
+  typedef status (*perform_func_type)(reactor_op*);
 
-  reactor_op(perform_func_type perform_func, func_type complete_func)
+  reactor_op(const asio::error_code& success_ec,
+      perform_func_type perform_func, func_type complete_func)
     : operation(complete_func),
+      ec_(success_ec),
+      cancellation_key_(0),
       bytes_transferred_(0),
       perform_func_(perform_func)
   {
