@@ -2,7 +2,7 @@
 // basic_logger.hpp
 // ~~~~~~~~~~~~~~~~
 //
-// Copyright (c) 2003-2016 Christopher M. Kohlhoff (chris at kohlhoff dot com)
+// Copyright (c) 2003-2023 Christopher M. Kohlhoff (chris at kohlhoff dot com)
 //
 // Distributed under the Boost Software License, Version 1.0. (See accompanying
 // file LICENSE_1_0.txt or copy at http://www.boost.org/LICENSE_1_0.txt)
@@ -34,13 +34,13 @@ public:
   /**
    * This constructor creates a logger.
    *
-   * @param io_service The io_service object used to locate the logger service.
+   * @param context The execution context used to locate the logger service.
    *
    * @param identifier An identifier for this logger.
    */
-  explicit basic_logger(asio::io_service& io_service,
+  explicit basic_logger(asio::execution_context& context,
       const std::string& identifier)
-    : service_(asio::use_service<Service>(io_service)),
+    : service_(asio::use_service<Service>(context)),
       impl_(service_.null())
   {
     service_.create(impl_, identifier);
@@ -52,12 +52,6 @@ public:
     service_.destroy(impl_);
   }
 
-  /// Get the io_service associated with the object.
-  asio::io_service& get_io_service()
-  {
-    return service_.get_io_service();
-  }
-
   /// Set the output file for all logger instances.
   void use_file(const std::string& file)
   {
