@@ -2,25 +2,23 @@
 // daytime_client.cpp
 // ~~~~~~~~~~~~~~~~~~
 //
-// Copyright (c) 2003-2016 Christopher M. Kohlhoff (chris at kohlhoff dot com)
+// Copyright (c) 2003-2023 Christopher M. Kohlhoff (chris at kohlhoff dot com)
 //
 // Distributed under the Boost Software License, Version 1.0. (See accompanying
 // file LICENSE_1_0.txt or copy at http://www.boost.org/LICENSE_1_0.txt)
 //
 
 #include <asio.hpp>
-#include <boost/bind.hpp>
+#include <boost/bind/bind.hpp>
 #include <iostream>
 #include "logger.hpp"
-#include "stream_socket_service.hpp"
 
-typedef asio::basic_stream_socket<asio::ip::tcp,
-    services::stream_socket_service<asio::ip::tcp> > debug_stream_socket;
+using asio::ip::tcp;
 
 char read_buffer[1024];
 
 void read_handler(const asio::error_code& e,
-    std::size_t bytes_transferred, debug_stream_socket* s)
+    std::size_t bytes_transferred, tcp::socket* s)
 {
   if (!e)
   {
@@ -30,19 +28,37 @@ void read_handler(const asio::error_code& e,
         boost::bind(read_handler, asio::placeholders::error,
           asio::placeholders::bytes_transferred, s));
   }
+  else
+  {
+    asio::execution_context& context = asio::query(
+        s->get_executor(), asio::execution::context);
+    services::logger logger(context, "read_handler");
+
+    std::string msg = "Read error: ";
+    msg += e.message();
+    logger.log(msg);
+  }
 }
 
-void connect_handler(const asio::error_code& e, debug_stream_socket* s)
+void connect_handler(const asio::error_code& e, tcp::socket* s)
 {
+  asio::execution_context& context = asio::query(
+      s->get_executor(), asio::execution::context);
+  services::logger logger(context, "connect_handler");
+
   if (!e)
   {
+    logger.log("Connection established");
+
     s->async_read_some(asio::buffer(read_buffer),
         boost::bind(read_handler, asio::placeholders::error,
           asio::placeholders::bytes_transferred, s));
   }
   else
   {
-    std::cerr << e.message() << std::endl;
+    std::string msg = "Unable to establish connection: ";
+    msg += e.message();
+    logger.log(msg);
   }
 }
 
@@ -56,25 +72,25 @@ int main(int argc, char* argv[])
       return 1;
     }
 
-    asio::io_service io_service;
+    asio::io_context io_context;
 
     // Set the name of the file that all logger instances will use.
-    services::logger logger(io_service, "");
+    services::logger logger(io_context, "");
     logger.use_file("log.txt");
 
     // Resolve the address corresponding to the given host.
-    asio::ip::tcp::resolver resolver(io_service);
-    asio::ip::tcp::resolver::query query(argv[1], "daytime");
-    asio::ip::tcp::resolver::iterator iterator = resolver.resolve(query);
+    tcp::resolver resolver(io_context);
+    tcp::resolver::results_type endpoints =
+      resolver.resolve(argv[1], "daytime");
 
     // Start an asynchronous connect.
-    debug_stream_socket socket(io_service);
-    asio::async_connect(socket, iterator,
+    tcp::socket socket(io_context);
+    asio::async_connect(socket, endpoints,
         boost::bind(connect_handler,
           asio::placeholders::error, &socket));
 
-    // Run the io_service until all operations have finished.
-    io_service.run();
+    // Run the io_context until all operations have finished.
+    io_context.run();
   }
   catch (std::exception& e)
   {
