@@ -2,7 +2,7 @@
 // timer.cpp
 // ~~~~~~~~~
 //
-// Copyright (c) 2003-2016 Christopher M. Kohlhoff (chris at kohlhoff dot com)
+// Copyright (c) 2003-2023 Christopher M. Kohlhoff (chris at kohlhoff dot com)
 //
 // Distributed under the Boost Software License, Version 1.0. (See accompanying
 // file LICENSE_1_0.txt or copy at http://www.boost.org/LICENSE_1_0.txt)
@@ -10,18 +10,17 @@
 
 #include <iostream>
 #include <asio.hpp>
-#include <boost/bind.hpp>
-#include <boost/date_time/posix_time/posix_time.hpp>
+#include <boost/bind/bind.hpp>
 
 void print(const asio::error_code& /*e*/,
-    asio::deadline_timer* t, int* count)
+    asio::steady_timer* t, int* count)
 {
   if (*count < 5)
   {
     std::cout << *count << std::endl;
     ++(*count);
 
-    t->expires_at(t->expires_at() + boost::posix_time::seconds(1));
+    t->expires_at(t->expiry() + asio::chrono::seconds(1));
     t->async_wait(boost::bind(print,
           asio::placeholders::error, t, count));
   }
@@ -29,10 +28,10 @@ void print(const asio::error_code& /*e*/,
 
 int main()
 {
-  asio::io_service io;
+  asio::io_context io;
 
   int count = 0;
-  asio::deadline_timer t(io, boost::posix_time::seconds(1));
+  asio::steady_timer t(io, asio::chrono::seconds(1));
   t.async_wait(boost::bind(print,
         asio::placeholders::error, &t, &count));
 
