@@ -2,7 +2,7 @@
 // echo_server.cpp
 // ~~~~~~~~~~~~~~~
 //
-// Copyright (c) 2003-2022 Christopher M. Kohlhoff (chris at kohlhoff dot com)
+// Copyright (c) 2003-2023 Christopher M. Kohlhoff (chris at kohlhoff dot com)
 //
 // Distributed under the Boost Software License, Version 1.0. (See accompanying
 // file LICENSE_1_0.txt or copy at http://www.boost.org/LICENSE_1_0.txt)
@@ -40,7 +40,7 @@ public:
             char data[128];
             for (;;)
             {
-              timer_.expires_from_now(std::chrono::seconds(10));
+              timer_.expires_after(std::chrono::seconds(10));
               std::size_t n = socket_.async_read_some(asio::buffer(data), yield);
               asio::async_write(socket_, asio::buffer(data, n), yield);
             }
@@ -59,7 +59,7 @@ public:
           {
             asio::error_code ignored_ec;
             timer_.async_wait(yield[ignored_ec]);
-            if (timer_.expires_from_now() <= std::chrono::seconds(0))
+            if (timer_.expiry() <= asio::steady_timer::clock_type::now())
               socket_.close();
           }
         }, asio::detached);
@@ -99,7 +99,12 @@ int main(int argc, char* argv[])
               std::make_shared<session>(io_context, std::move(socket))->go();
             }
           }
-        }, asio::detached);
+        },
+        [](std::exception_ptr e)
+        {
+          if (e)
+            std::rethrow_exception(e);
+        });
 
     io_context.run();
   }
