@@ -2,7 +2,7 @@
 // server.cpp
 // ~~~~~~~~~~
 //
-// Copyright (c) 2003-2016 Christopher M. Kohlhoff (chris at kohlhoff dot com)
+// Copyright (c) 2003-2023 Christopher M. Kohlhoff (chris at kohlhoff dot com)
 //
 // Distributed under the Boost Software License, Version 1.0. (See accompanying
 // file LICENSE_1_0.txt or copy at http://www.boost.org/LICENSE_1_0.txt)
@@ -10,7 +10,7 @@
 
 #include "asio.hpp"
 #include <algorithm>
-#include <boost/bind.hpp>
+#include <boost/bind/bind.hpp>
 #include <iostream>
 #include <list>
 #include "handler_allocator.hpp"
@@ -18,10 +18,10 @@
 class session
 {
 public:
-  session(asio::io_service& ios, size_t block_size)
-    : io_service_(ios),
-      strand_(ios),
-      socket_(ios),
+  session(asio::io_context& ioc, size_t block_size)
+    : io_context_(ioc),
+      strand_(ioc.get_executor()),
+      socket_(ioc),
       block_size_(block_size),
       read_data_(new char[block_size]),
       read_data_length_(0),
@@ -51,7 +51,7 @@ public:
     {
       ++op_count_;
       socket_.async_read_some(asio::buffer(read_data_, block_size_),
-          strand_.wrap(
+          asio::bind_executor(strand_,
             make_custom_alloc_handler(read_allocator_,
               boost::bind(&session::handle_read, this,
                 asio::placeholders::error,
@@ -59,7 +59,7 @@ public:
     }
     else
     {
-      io_service_.post(boost::bind(&session::destroy, this));
+      asio::post(io_context_, boost::bind(&session::destroy, this));
     }
   }
 
@@ -76,12 +76,12 @@ public:
         op_count_ += 2;
         std::swap(read_data_, write_data_);
         async_write(socket_, asio::buffer(write_data_, read_data_length_),
-            strand_.wrap(
+            asio::bind_executor(strand_,
               make_custom_alloc_handler(write_allocator_,
                 boost::bind(&session::handle_write, this,
                   asio::placeholders::error))));
         socket_.async_read_some(asio::buffer(read_data_, block_size_),
-            strand_.wrap(
+            asio::bind_executor(strand_,
               make_custom_alloc_handler(read_allocator_,
                 boost::bind(&session::handle_read, this,
                   asio::placeholders::error,
@@ -90,7 +90,7 @@ public:
     }
 
     if (op_count_ == 0)
-      io_service_.post(boost::bind(&session::destroy, this));
+      asio::post(io_context_, boost::bind(&session::destroy, this));
   }
 
   void handle_write(const asio::error_code& err)
@@ -105,12 +105,12 @@ public:
         op_count_ += 2;
         std::swap(read_data_, write_data_);
         async_write(socket_, asio::buffer(write_data_, read_data_length_),
-            strand_.wrap(
+            asio::bind_executor(strand_,
               make_custom_alloc_handler(write_allocator_,
                 boost::bind(&session::handle_write, this,
                   asio::placeholders::error))));
         socket_.async_read_some(asio::buffer(read_data_, block_size_),
-            strand_.wrap(
+            asio::bind_executor(strand_,
               make_custom_alloc_handler(read_allocator_,
                 boost::bind(&session::handle_read, this,
                   asio::placeholders::error,
@@ -119,7 +119,7 @@ public:
     }
 
     if (op_count_ == 0)
-      io_service_.post(boost::bind(&session::destroy, this));
+      asio::post(io_context_, boost::bind(&session::destroy, this));
   }
 
   static void destroy(session* s)
@@ -128,8 +128,8 @@ public:
   }
 
 private:
-  asio::io_service& io_service_;
-  asio::io_service::strand strand_;
+  asio::io_context& io_context_;
+  asio::strand<asio::io_context::executor_type> strand_;
   asio::ip::tcp::socket socket_;
   size_t block_size_;
   char* read_data_;
@@ -144,10 +144,10 @@ private:
 class server
 {
 public:
-  server(asio::io_service& ios, const asio::ip::tcp::endpoint& endpoint,
+  server(asio::io_context& ioc, const asio::ip::tcp::endpoint& endpoint,
       size_t block_size)
-    : io_service_(ios),
-      acceptor_(ios),
+    : io_context_(ioc),
+      acceptor_(ioc),
       block_size_(block_size)
   {
     acceptor_.open(endpoint.protocol());
@@ -160,7 +160,7 @@ public:
 
   void start_accept()
   {
-    session* new_session = new session(io_service_, block_size_);
+    session* new_session = new session(io_context_, block_size_);
     acceptor_.async_accept(new_session->socket(),
         boost::bind(&server::handle_accept, this, new_session,
           asio::placeholders::error));
@@ -181,7 +181,7 @@ public:
   }
 
 private:
-  asio::io_service& io_service_;
+  asio::io_context& io_context_;
   asio::ip::tcp::acceptor acceptor_;
   size_t block_size_;
 };
@@ -197,25 +197,25 @@ int main(int argc, char* argv[])
     }
 
     using namespace std; // For atoi.
-    asio::ip::address address = asio::ip::address::from_string(argv[1]);
+    asio::ip::address address = asio::ip::make_address(argv[1]);
     short port = atoi(argv[2]);
     int thread_count = atoi(argv[3]);
     size_t block_size = atoi(argv[4]);
 
-    asio::io_service ios;
+    asio::io_context ioc;
 
-    server s(ios, asio::ip::tcp::endpoint(address, port), block_size);
+    server s(ioc, asio::ip::tcp::endpoint(address, port), block_size);
 
     // Threads not currently supported in this test.
     std::list<asio::thread*> threads;
     while (--thread_count > 0)
     {
       asio::thread* new_thread = new asio::thread(
-          boost::bind(&asio::io_service::run, &ios));
+          boost::bind(&asio::io_context::run, &ioc));
       threads.push_back(new_thread);
     }
 
-    ios.run();
+    ioc.run();
 
     while (!threads.empty())
     {
