@@ -2,7 +2,7 @@
 // signal_set.cpp
 // ~~~~~~~~~~~~~~
 //
-// Copyright (c) 2003-2022 Christopher M. Kohlhoff (chris at kohlhoff dot com)
+// Copyright (c) 2003-2023 Christopher M. Kohlhoff (chris at kohlhoff dot com)
 //
 // Distributed under the Boost Software License, Version 1.0. (See accompanying
 // file LICENSE_1_0.txt or copy at http://www.boost.org/LICENSE_1_0.txt)
@@ -66,6 +66,9 @@ void test()
     set1.add(1);
     set1.add(1, ec);
 
+    set1.add(1, signal_set::flags::dont_care);
+    set1.add(1, signal_set::flags::dont_care, ec);
+
     set1.remove(1);
     set1.remove(1, ec);
 
